/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _LEVEQUE_CORRECTOR_H
#define _LEVEQUE_CORRECTOR_H

// original design            : Phil Colella
// original implementation    : Brian Van Straalen
// implementation date        : Wed Apr 28, 1999
// transmogrification to 0.99 : Dave Modiano
// transmogrification date    : 10 Jan 2000

#include "SPACE.H"
#include "REAL.H"
#include "IntVectSet.H"
#include "Vector.H"
#include "FArrayBox.H"
#include "Tuple.H"
#include "LayoutData.H"

class DataIndex;
class DisjointBoxLayout;

/// LeVeque corrector
/**
 */
class LevequeCorrector
{
public:
  ///
  LevequeCorrector();

  ///
  virtual ~LevequeCorrector();

  /// Define the object
  void define(const DisjointBoxLayout& a_ba,
              const Box&               a_dProblem,
              const Real&              a_dx,
              const int                a_numFLuxes);


  /// Define the object
  void define(const DisjointBoxLayout& a_ba,
              const ProblemDomain&     a_dProblem,
              const Real&              a_dx,
              const int                a_numFLuxes);

  /// Correct the boundary
  void correctBoundary(FArrayBox&       a_UPatch,
                       FArrayBox&       a_flux,
                       Real             a_dt,
                       Real             a_dx,
                       int              a_fluxDir,
                       const DataIndex& a_patchIndex) const ;

protected:
  // wipe out data structures and release memory, set
  // object to null construction state.
  void clearAll();

private:
  bool                   m_isDefined;
  int                    m_numFluxes;
  LayoutData<IntVectSet> m_lo_masks[SpaceDim];
  LayoutData<IntVectSet> m_hi_masks[SpaceDim];
};

#endif
