/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

inline void loHiCenter(Box a_inBox,
                       const ProblemDomain& a_DProblem,
                       const int& a_sweepdir,
                       Box& a_loBox,
                       Box& a_hiBox,
                       Box& a_centerBox,
                       Box& a_Domain,
                       int& a_haslo,
                       int& a_hashi);

inline void loHiCenter(Box a_inBox,
                       const Box& a_DProblem,
                       const int& a_sweepdir,
                       Box& a_loBox,
                       Box& a_hiBox,
                       Box& a_centerBox,
                       Box& a_Domain,
                       int& a_haslo,
                       int& a_hashi)
{
  ProblemDomain physdomain(a_DProblem);

  loHiCenter(a_inBox,physdomain,a_sweepdir,a_loBox,
             a_hiBox,a_centerBox,a_Domain,a_haslo,a_hashi);
}

inline void loHiCenter(Box a_inBox,
                       const ProblemDomain& a_DProblem,
                       const int& a_sweepdir,
                       Box& a_loBox,
                       Box& a_hiBox,
                       Box& a_centerBox,
                       Box& a_Domain,
                       int& a_haslo,
                       int& a_hashi)
{
  a_inBox &= a_DProblem;
  Box tmp = a_inBox;

  tmp.shift(a_sweepdir,1);
  tmp &= a_DProblem;
  tmp.shift(a_sweepdir,-1);

  a_Domain = a_inBox;
  a_Domain.grow(a_sweepdir,-1);
  a_centerBox = a_Domain;

  if (tmp != a_inBox)
  {
    a_hashi = 1;
    a_hiBox = adjCellHi(tmp,a_sweepdir);
    a_Domain.growHi(a_sweepdir,1);
  }
  else
  {
    a_hashi = 0;
  }

  tmp = a_inBox;
  tmp.shift(a_sweepdir,-1);
  tmp &= a_DProblem;
  tmp.shift(a_sweepdir,1);

  if (tmp != a_inBox)
  {
    a_haslo = 1;
    a_loBox = adjCellLo(tmp,a_sweepdir);
    a_Domain.growLo(a_sweepdir,1);
  }
  else
  {
    a_haslo = 0;
  }

  assert(a_Domain.contains(a_centerBox));

  if (a_haslo == 1)
  {
    assert(a_Domain.contains(a_loBox));
  }

  if (a_hashi == 1)
  {
    assert(a_Domain.contains(a_hiBox));
  }
}
