C -*- Mode: Fortran; -*-
c-----------------------------------------------------------------
c       Ravi Samtaney
c       Copyright 2001
c       Princeton Plasma Physics Laboratory
c       All Rights Reserved
c-----------------------------------------------------------------
c       $Log: init3.F,v $
c       Revision 1.1  2002/06/13 22:16:18  samtaney
c       Original unsplit 8-wave Ideal MHD source (includes B filter).
c
c-----------------------------------------------------------------
c Subroutine to set up the initial conditions
c---------------------------------------------------------------------------
c
      subroutine RMInitialConditions(ux,
     &                     iulo,iuhi,julo,juhi,
     &                     ilo,ihi,jlo,jhi,
     &                     klo,khi,
     &                     adx,ady,adz)
c----------------------------------------------------------------------
	use properties
	use mesh
c=======Declarations=========
	integerilo,ihi,jlo,jhi,klo,khi
	integeriulo,iuhi,julo,juhi
	real*8 ux(iulo:iuhi,
     &                        julo:juhi,
     &                        1:1,nvar)
c     &                        klo-nghost:khi+nghost,nvar)
c
	integer i,j,k,l
        real*8 xl,yl,xr,yr
        real*8 rho,press, Bi, Bj, Bk, u,v,w
        real*8 adx,ady,adz
        real*8 dxi,dyi,xdriver,xinterface
c
	integer i,j,k
	character*24 fdate
        real*8 x,y,z
        real*8 pi, kx,ky,lamda
        real*8 p1,rho1,u1,amach,gas1,gas2
c
	gamma=1.4D0
	rgas=1.D0
	wmol=1.D0
	mu=1.D-03
	eta=1.D-03
	kappa=1.D-03
c
c        xl=-1.0D0
        xl=0.0D0
        xr=2.D0
c        yl=-1.D0
        yl=0.D0
c        yr=0.5D0
        yr=2.0D0
c
	pi=4.D0*datan(1.D0)
        dxi=0.5D0/adx
        dyi=0.5D0/ady
c
        rho=1.D0
        press=1.D0
	u=0.D0
	v=0.D0
	w=0.D0
c
	kx=3.D0*pi/2.D0
	ky=pi
c        
c        xdriver=-0.2D0
c        xinterface=0.D0
        xdriver=1.0D0
        xinterface=0.5D0
        Bi=0.00D0
        Bj=0.0D0
        Bk=0.D0
c     Setup interface
        do k=klo,khi,1
           do j=jlo,jhi,1
              y = yl+(j - 0.5D0)*ady;
              do i=ilo,ihi,1
                 x = xl+(i - 0.5D0)*adx;
c                 if(y-yl.lt.x-xinterface) then
c                 if(x-xinterface.gt.0.D0) then
c                    rho=5.D0
                    rho=1.D0
c                 else
c                    rho=1.D0
c                 endif
                 ux(i,j,k,1)=rho
                 ux(i,j,k,2)=rho*u
                 ux(i,j,k,3)=rho*v
                 ux(i,j,k,4)=rho*w
                 ux(i,j,k,5)=Bi
                 ux(i,j,k,6)=Bj
                 ux(i,j,k,7)=Bk
                 ux(i,j,k,8)=press/(gamma-1.D0)+
     &                0.5*rho*(u*u+v*v+w*w)+0.5D0*(bi*bi+bj*bj+bk*bk)
              enddo
           enddo
        enddo
c
c     Setup Gas dynamic shock - assume zero flow ahead of shock
        amach=1.2D0
        rho=1.D0
        press=1.D0
        p1=press*(1.0D0+2.D0*gamma/(gamma+1.D0)*(amach*amach-1.D0))
        gas1=(gamma-1.D0)/(gamma+1.D0)
        gas2=2.D0*gamma/(gamma+1.D0)
        rho1=rho*(p1/press+gas1)/(gas1*p1/press+1.D0)
        u1=((rho1-rho)*dsqrt(gamma*press/rho)*amach)/rho1
         do k=klo,khi,1
           do j=jlo,jhi,1
              y = yl+(j - 0.5D0)*ady;
              do i=ilo,ihi,1
                 x = xl+(i - 0.5D0)*adx;
                 if(x.lt.xdriver) then
                    ux(i,j,k,1)=rho1
                    ux(i,j,k,2)=rho1*u1
                    ux(i,j,k,3)=0.D0
                    ux(i,j,k,4)=0.D0
                    ux(i,j,k,5)=Bi
                    ux(i,j,k,6)=Bj
                    ux(i,j,k,7)=Bk
                    ux(i,j,k,8)=p1/(gamma-1.D0)+
     &                   0.5*rho1*(u1*u1)+0.5D0*(bi*bi+bj*bj+bk*bk)
                 endif
              enddo
           enddo
        enddo
c
      return
      end







