C     -*- Mode: Fortran; -*-
c-----------------------------------------------------------------------
c     Ravi Samtaney
c     Copyright 2002
c     Princeton Plasma Physics Laboratory
c     All Rights Reserved
c-----------------------------------------------------------------------
c     $Log: normalpred.F,v $
c     Revision 1.4  2002/07/19 13:41:23  samtaney
c     Fixed order of arguments.
c
c     Revision 1.3  2002/07/10 13:18:14  samtaney
c     Fixed typos - jlo,jhi were used instead of klo,khi in a call to constructlrstates.
c
c     Revision 1.2  2002/07/09 19:00:59  samtaney
c     Intermediate commit of files towards a single 2D/3D Ideal MHD code.
c
c     Revision 1.1  2002/06/13 22:16:18  samtaney
c     Original unsplit 8-wave Ideal MHD source (includes B filter).
c
c-----------------------------------------------------------------------
c      subroutine NormalPredMHD(ul,ur,ux,dudx,
c     &     iullo,iulhi,jullo,julhi,kullo,kulhi,
c     &     iurlo,iurhi,jurlo,jurhi,kurlo,kurhi,
      subroutine NormalPredMHD(ur,ul,ux,dudx,
     &     iurlo,iurhi,jurlo,jurhi,kurlo,kurhi,
     &     iullo,iulhi,jullo,julhi,kullo,kulhi,
     &     iuxlo,iuxhi,juxlo,juxhi,kuxlo,kuxhi,
     &     idudxlo,idudxhi,jdudxlo,jdudxhi,kdudxlo,kdudxhi,
     &     ilo,ihi,jlo,jhi,klo,khi,
     &     direction,llo,lhi,dtodx)
c
      implicit none
c
#include "EosCommon.fh"
c     
c     Incoming arguments
      integer iullo,iulhi,jullo,julhi,kullo,kulhi,
     &     iurlo,iurhi,jurlo,jurhi,kurlo,kurhi,
     &     iuxlo,iuxhi,juxlo,juxhi,kuxlo,kuxhi,
     &     idudxlo,idudxhi,jdudxlo,jdudxhi,kdudxlo,kdudxhi,
     &     ilo,ihi,jlo,jhi,klo,khi
      integer llo,lhi,direction
c     
      real*8 ul(iullo:iulhi,jullo:julhi,kullo:kulhi,8)
      real*8 ur(iurlo:iurhi,jurlo:jurhi,kurlo:kurhi,8)
      real*8 ux(iuxlo:iuxhi,juxlo:juxhi,kuxlo:kuxhi,8)
      real*8 dudx(idudxlo:idudxhi,jdudxlo:jdudxhi,
     &     kdudxlo:kdudxhi,8)
c     
c     Local variables
      integer i,j,k,l,m
      real*8 vx(llo:lhi,8)
      real*8 dvdx(llo:lhi,8)
      real*8 vl(llo:lhi,8)
      real*8 vr(llo:lhi,8)
      real*8 dtodx
c     
c$$$  write(6,*) 'Normalpred', iullo,iulhi,jullo,julhi,
c$$$  &                         iurlo,iurhi,jurlo,jurhi,
c$$$  &                         iuxlo,iuxhi,juxlo,juxhi,
c$$$  &                         idudxlo,idudxhi,jdudxlo,jdudxhi,
c$$$  &                         ilo,ihi,jlo,jhi,klo,khi
c     
      if(direction.eq.0) then
         do k=klo,khi,1
            do j=jlo,jhi,1
               do i=ilo,ihi,1
                  do m=1,8,1
                     vx(i,m)=ux(i,j,k,m)
                     dvdx(i,m)=dudx(i,j,k,m)
                  enddo
               enddo
               call ConstructLRStates(vx,vl,vr,dvdx,ilo,ihi,direction,
     &              dtodx)
               do i=ilo,ihi,1
                  do m=1,8,1
                     ul(i,j,k,m)=vl(i,m)
                     ur(i,j,k,m)=vr(i,m)
                  enddo
              enddo
            enddo
         enddo
      elseif (direction.eq.1) then
         do k=klo,khi,1
            do i=ilo,ihi,1
               do j=jlo,jhi,1
                  do m=1,8,1
                     vx(j,m)=ux(i,j,k,m)
                     dvdx(j,m)=dudx(i,j,k,m)
                  enddo
               enddo
               call RemapToLocalCoords(vx,jlo,jhi,2)
               call RemapToLocalCoords(dvdx,jlo,jhi,2)
               call ConstructLRStates(vx,vl,vr,dvdx,jlo,jhi,direction,
     &              dtodx)
               call RemapToGlobalCoords(vl,jlo,jhi,2)
               call RemapToGlobalCoords(vr,jlo,jhi,2)
               do j=jlo,jhi,1
                  do m=1,8,1
                     ul(i,j,k,m)=vl(j,m)
                     ur(i,j,k,m)=vr(j,m)
                  enddo
               enddo
            enddo
         enddo
      elseif (direction.eq.2) then
         do j=jlo,jhi,1
            do i=ilo,ihi,1
               do k=klo,khi,1
                  do m=1,8,1
                     vx(k,m)=ux(i,j,k,m)
                     dvdx(k,m)=dudx(i,j,k,m)
                  enddo
               enddo
               call RemapToLocalCoords(vx,klo,khi,3)
               call RemapToLocalCoords(dvdx,klo,khi,3)
               call ConstructLRStates(vx,vl,vr,dvdx,klo,khi,direction,
     &              dtodx)
               call RemapToGlobalCoords(vl,klo,khi,3)
               call RemapToGlobalCoords(vr,klo,khi,3)
               do k=klo,khi,1
                  do m=1,8,1
                     ul(i,j,k,m)=vl(k,m)
                     ur(i,j,k,m)=vr(k,m)
                  enddo
               enddo
            enddo
         enddo
c     
      else
         write(6,*) 'FATAL ERROR: direcion !=0,1,2'
c     
      endif
c     
      return
      end
c
c-----------------------------------------------------------------------
