/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// DFMartin, Sun, May 5, 2002

#ifndef _BiCGStabSmoother_H_
#define _BiCGStabSmoother_H_

#include <iostream>
#include <math.h>
#include <assert.h>
#include <stdlib.h>
#include <REAL.H>
#include <IntVect.H>
#include <Box.H>
#include <DisjointBoxLayout.H>
#include <LevelData.H>
#include <FArrayBox.H>
#include <ProblemDomain.H>
#include "BaseBottomSmoother.H"

/// Implements BiCGStab as a bottom smoother for LevelOp-derived classes
/** The BiCGStabSmoother class implements the BiCGStab algorithm for 
    LevelOp-derived classes
*/ 
class BiCGStabSmoother : public BaseBottomSmoother
{

public:

  /// constructor, destructor


  /// default constructor
  BiCGStabSmoother();

  ///  destructor
  virtual ~BiCGStabSmoother();

  ///
  /** this gets around the "no virtual constructor" rule
   */
  virtual BaseBottomSmoother* new_bottomSmoother() const;

  /// set maximum number of BiCGStab iterations
  /** set max number of BiCGStab iterations
   */
  void setMaxIter(int a_max_iter);

  /// set solver tolerance
  /** set solver tolerance (amount residual must be decreased, etc)
   */
  void setSolverTol(Real a_solver_tol);
  
  /// set verbosity level
  /** 
      set whether i/o is done when solving (default is false)
  */
  void setVerbose(bool a_verbose);


  /// do smoothing
  /** performs BiCGStab smoothing of L(a_phi) = a_rhs; L is the operator
      defined in a_levelop_ptr, and the preconditioner is
      a_levelop_ptr->smooth
  */
  virtual void doBottomSmooth(LevelData<FArrayBox>& a_phi,
                              const LevelData<FArrayBox>& a_rhs,
                              LevelOp* a_levelop_ptr);

protected:

  int m_maxIter;

  Real m_solver_tol;
  
  // small parameter to check for division by zero
  Real m_small;

  // default is false
  bool m_verbose;
  
};

#endif
  
  


