/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _CFIVS_H_
#define _CFIVS_H_

#include <iostream>
#include <cmath>
#include <assert.h>
#include <cstdlib>
#include "REAL.H"
#include "IntVect.H"
#include "Box.H"
#include "LoHiSide.H"
#include "DisjointBoxLayout.H"
#include "IntVectSet.H"
#include "ProblemDomain.H"

///Class to get fine intvectset at CF interface
/** 
Class to get fine intvectset at cf interface.
This class should be considered internal to AMRSolver and should
not be considered part of the Chombo API.
*/
class CFIVS
{

private:

protected:
  //for internal use
  void  setDefaultValues();

  //the following variables are mainly for self-identification

  /* fine intvects which need to be interpolated */
  IntVectSet fiinterp_ivs;  

  bool isdefined;

public:

  ///
  bool isDefined() const;

  ///
  /** Get fine intvects which need to be interpolated.
      This will be empty if isEmpty() returns true*/
  const IntVectSet& getFineIVS() const ;

  /// 
  CFIVS();

  /// 
  ~CFIVS();

  /// 
  CFIVS(
        const Box& Domain, 
        const Box& box_in,
        const DisjointBoxLayout& Levboxes, 
        int Direction, 
        Side::LoHiSide hiorlo);


  /// 
  CFIVS(
        const ProblemDomain& Domain, 
        const Box& box_in,
        const DisjointBoxLayout& Levboxes, 
        int Direction, 
        Side::LoHiSide hiorlo);

  ///
  void define(
              const Box& Domain, 
              const Box& box_in,
              const DisjointBoxLayout& fine_boxes, 
              int Direction, 
              Side::LoHiSide hiorlo);

  ///
  void define(
              const ProblemDomain& Domain, 
              const Box& box_in,
              const DisjointBoxLayout& fine_boxes, 
              int Direction, 
              Side::LoHiSide hiorlo);

private:
  void operator= (const CFIVS& levcfs_in) {};
  CFIVS(const CFIVS& levcfs_in) {};

};
#endif
