/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "DisjointBoxLayout.H"
#include "LevelData.H"
#include "BaseFab.H"
#include "REAL.H"
#include "DataIterator.H"
#include "LevelMGF_F.H"

#include "MGInterp.H"

MGInterp::MGInterp()
  :
  is_defined(false)
{
}


MGInterp::~MGInterp()
{
}


MGInterp::MGInterp(const DisjointBoxLayout& a_fine_domain,
                       int a_numcomps,
                       int a_ref_ratio,
                       const Box& a_problem_domain)
  :
  is_defined(false)
{
  ProblemDomain fine_physdomain(a_problem_domain);
  define(a_fine_domain, a_numcomps, a_ref_ratio, fine_physdomain);
}


MGInterp::MGInterp(const DisjointBoxLayout& a_fine_domain,
                       int a_numcomps,
                       int a_ref_ratio,
                       const ProblemDomain& a_problem_domain)
  :
  is_defined(false)
{
  define(a_fine_domain, a_numcomps, a_ref_ratio, a_problem_domain);
}


void
MGInterp::define(const DisjointBoxLayout& a_fine_domain,
                 int a_numcomps,
                 int a_ref_ratio,
                 const Box& a_problem_domain)
{
  ProblemDomain physdomain(a_problem_domain);
  define(a_fine_domain, a_numcomps, a_ref_ratio, physdomain);
}



void
MGInterp::define(const DisjointBoxLayout& a_fine_domain,
                 int a_numcomps,
                 int a_ref_ratio,
                 const ProblemDomain& a_problem_domain)
{
  m_ref_ratio = a_ref_ratio;
  m_problem_domain = a_problem_domain;
  m_grids = a_fine_domain;
//
// create the work array 
  DisjointBoxLayout coarsened_fine_domain;
  coarsen ( coarsened_fine_domain, 
            a_fine_domain, 
            m_ref_ratio );
  m_coarsened_fine_data.define ( coarsened_fine_domain, 
                                 a_numcomps, 
                                 IntVect::TheZeroVector());
  is_defined = true;
}



// interpolate from coarse level to fine level
void 
MGInterp::interpToFine(LevelData<FArrayBox>& a_fine_data,
                       const LevelData<FArrayBox>& a_coarse_data)
{
  assert(is_defined);
  a_coarse_data.copyTo(a_coarse_data.interval(),
                       m_coarsened_fine_data,
                       m_coarsened_fine_data.interval() );

  DataIterator dit = m_grids.dataIterator();

  for (dit.begin(); dit.ok(); ++dit)
  {
    FArrayBox& fine = a_fine_data[dit()];
    const FArrayBox& coarsened_fine = m_coarsened_fine_data[dit()];
    Box crseBox = coarsened_fine.box();
    Box fineBox = m_grids.get(dit());
    fineBox.coarsen(m_ref_ratio);
    assert(fineBox == crseBox);

    Box nrefbox(IntVect::TheZeroVector(), 
                (m_ref_ratio-1)*IntVect::TheUnitVector());
    FORT_INTERPMG(CHF_FRA(fine),
                  CHF_FRA(coarsened_fine),
                  CHF_BOX(crseBox),
                  CHF_CONST_INT(m_ref_ratio),
                  CHF_BOX(nrefbox)
                  );
  }
}


bool
MGInterp::isDefined() const
{
  return ( is_defined );
}
