/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "BoxLayoutData.H"

#ifdef MPI
#include <mpi.h>
#endif

Real norm(const BoxLayoutData<FArrayBox>& a_layout, 
          const Interval& interval,
          const int& p)
{
  const BoxLayout validRegion = a_layout.boxLayout();
  DataIterator it = validRegion.dataIterator();
  Real norm = 0;
  if(p == 0)  // max norm
    {
      for(;it.ok(); ++it)
        {
          norm = Max(norm, a_layout[it()].norm(validRegion[it()],
                                               0,
                                               interval.begin(),
                                               interval.size()));
        }
#     ifdef MPI
      Real recv;
      int result = MPI_Allreduce(&norm, &recv, 1, MPI_CH_REAL,
                                 MPI_MAX, Chombo_MPI::comm);
      if(result != MPI_SUCCESS){ //bark!!!
        MayDay::Error("sorry, but I had a communcation error on norm");
      }
      norm = recv;
#     endif
    }
  else if(p == 1) // abs sum norm
    {
      for(;it.ok(); ++it)
        {
          norm += a_layout[it()].norm(validRegion[it()],
                                      1,
                                      interval.begin(),
                                      interval.size());
        }
#     ifdef MPI
      Real recv;
      int result = MPI_Allreduce(&norm, &recv, 1, MPI_CH_REAL,
                                 MPI_SUM, Chombo_MPI::comm);
      if(result != MPI_SUCCESS){ //bark!!!
        MayDay::Error("sorry, but I had a communcation error on norm");
      }
      norm = recv;
#     endif
    }
  else
    {
      for(;it.ok(); ++it)
        {
          norm+=a_layout[it()].sumPow(validRegion[it()],
                                      p,
                                      interval.begin(),
                                      interval.size());
        }
#     ifdef MPI
      Real recv;
      int result = MPI_Allreduce(&norm, &recv, 1, MPI_CH_REAL,
                                 MPI_SUM, Chombo_MPI::comm);
      if(result != MPI_SUCCESS){ //bark!!!
        MayDay::Error("sorry, but I had a communcation error on norm");
      }
      norm = recv;
#     endif
      Real invpwr = 1.0/p;
      norm = pow(norm, invpwr);
    }

  return norm;
}

template <>
BaseFab<int>* DefaultDataFactory<BaseFab<int> >::create(const Box& box, 
                                                        int ncomps,
                                                        const DataIndex& a_datInd) const
{
  return new BaseFab<int>(box, ncomps);
}

template <>
FArrayBox* DefaultDataFactory<FArrayBox>::create(const Box& box, int ncomps,
                                          const DataIndex& a_datInd) const
{
  return new FArrayBox(box, ncomps);
}
