/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef LDF_H
#define LDF_H

#ifndef WRAPPER
#include "LevelData.H"
#include "FArrayBox.H"
#include "RefCountedPtr.H"
#endif

#include "Interval.H"

class LDF
{
public:
  LDF()
  {
	m_ptr = new LevelData<FArrayBox>();}

  LDF(LevelData<FArrayBox>* newPtr)
  {
	m_ptr = RefCountedPtr<LevelData<FArrayBox> >(newPtr);}

  void define(const DisjointBoxLayout& dp, int comps,
			  const IntVect& ghost) 
  {
	m_ptr->define(dp, comps, ghost);}

  ~LDF(){;}

  void exchange(const Interval& comps)
  {
	m_ptr->exchange(comps);}

  void copyTo(const Interval& srcComps, 
			  LDF& dest,
			  const Interval& destComps) const
  {
	m_ptr-> copyTo(srcComps, *(dest.m_ptr), destComps);}

  IntVect ghostVect() const {return m_ptr->ghostVect();}

  const DisjointBoxLayout& getBoxes() const {return m_ptr->getBoxes();}

  const DisjointBoxLayout& disjoiuntBoxLayout() const {return m_ptr->getBoxes();}

  Interval interval() const {return m_ptr->interval();}

  DataIterator dataIterator() const {return m_ptr->dataIterator();}

  Box box(const DataIndex& index) const { return m_ptr->box(index);}

  FArrayBox& get(const DataIndex& index) { return m_ptr->operator[](index);}

  const LevelData<FArrayBox>* dataPtr() const {return m_ptr;}

  void operator=(LevelData<FArrayBox>* newPtr){
	m_ptr = RefCountedPtr<LevelData<FArrayBox> >(newPtr);}

private:
  RefCountedPtr<LevelData<FArrayBox> > m_ptr;
};

#endif
